package com.ebei.screen.common.exception;

import cn.hutool.core.util.StrUtil;
import com.ebei.screen.common.constants.BusinessErrorCode;
import com.ebei.screen.common.response.ResponseBean;
import com.ebei.screen.common.response.ResponseBuilder;
import com.ebei.screen.common.util.LeviUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.method.annotation.MethodArgumentTypeMismatchException;
import org.springframework.web.multipart.MultipartException;
import org.springframework.web.multipart.support.MissingServletRequestPartException;
import org.springframework.web.servlet.NoHandlerFoundException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.UnexpectedTypeException;
import java.sql.SQLException;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 全局异常处理器
 *
 * @author Levi
 */
@Configuration
@ControllerAdvice(annotations = {RestController.class, Controller.class})
@ResponseBody
@Slf4j
public class MyExceptionHandler {

    @ExceptionHandler(MyException.class)
    public ResponseBean<?> handleRRException(MyException e) {
        log.info(LeviUtils.getExceptionInfo(e), e);
        return ResponseBuilder.fail(e.getCode(), e.getMessage());
    }

    @ExceptionHandler(HttpMessageNotReadableException.class)
    public ResponseBean<?> httpMessageNotReadableException(HttpMessageNotReadableException ex, HttpServletRequest request) {
        log.warn("HttpMessageNotReadableException:", ex);
        String message = ex.getMessage();
        if (StrUtil.containsAny(message, "Could not read document:")) {
            String msg = String.format("无法正确的解析json类型的参数：%s", StrUtil.subBetween(message, "Could not read document:", " at "));
            return ResponseBuilder.fail(msg);
        }
        return ResponseBuilder.fail(BusinessErrorCode.PARAM_EX.getCode(), BusinessErrorCode.PARAM_EX.getMessage());
    }

    @ExceptionHandler(BindException.class)
    public ResponseBean<?> bindException(BindException ex, HttpServletRequest request) {
        log.warn("BindException:", ex);
        try {
            String msgs = ex.getBindingResult().getFieldError().getDefaultMessage();
            if (StrUtil.isNotEmpty(msgs)) {
                return ResponseBuilder.fail(BusinessErrorCode.PARAM_EX.getCode(), BusinessErrorCode.PARAM_EX.getMessage());
            }
        } catch (Exception ee) {
        }
        StringBuilder msg = new StringBuilder();
        List<FieldError> fieldErrors = ex.getFieldErrors();
        fieldErrors.forEach((oe) ->
                msg.append("参数:[").append(oe.getObjectName())
                        .append(".").append(oe.getField())
                        .append("]的传入值:[").append(oe.getRejectedValue()).append("]与预期的字段类型不匹配.")
        );
        return ResponseBuilder.fail(BusinessErrorCode.PARAM_EX.getCode(), BusinessErrorCode.PARAM_EX.getMessage());
    }

    @ExceptionHandler(NoHandlerFoundException.class)
    public ResponseBean<?> handlerNoFoundException(Exception e) {
        log.info(LeviUtils.getExceptionInfo(e), e);
        return ResponseBuilder.fail(404, "路径不存在 请检查路径是否正确");
    }

    @ExceptionHandler(DuplicateKeyException.class)
    public ResponseBean<?> handleDuplicateKeyException(DuplicateKeyException e) {
        log.info(LeviUtils.getExceptionInfo(e), e);
        return ResponseBuilder.fail("数据库中已存在该记录");
    }

    @ExceptionHandler(MethodArgumentTypeMismatchException.class)
    public ResponseBean<?> methodArgumentTypeMismatchException(MethodArgumentTypeMismatchException ex, HttpServletRequest request) {
        log.warn("MethodArgumentTypeMismatchException:", ex);
        MethodArgumentTypeMismatchException eee = (MethodArgumentTypeMismatchException) ex;
        StringBuilder msg = new StringBuilder("参数：[").append(eee.getName())
                .append("]的传入值：[").append(eee.getValue())
                .append("]与预期的字段类型：[").append(eee.getRequiredType().getName()).append("]不匹配");
        return ResponseBuilder.fail(BusinessErrorCode.PARAM_EX.getCode(), BusinessErrorCode.PARAM_EX.getMessage());
    }

    @ExceptionHandler(IllegalStateException.class)
    public ResponseBean<?> illegalStateException(IllegalStateException ex, HttpServletRequest request) {
        log.warn("IllegalStateException:", ex);
        return ResponseBuilder.fail(BusinessErrorCode.ILLEGALA_ARGUMENT_EX.getCode(), BusinessErrorCode.ILLEGALA_ARGUMENT_EX.getMessage());
    }

    @ExceptionHandler(MissingServletRequestParameterException.class)
    public ResponseBean<?> missingServletRequestParameterException(MissingServletRequestParameterException ex, HttpServletRequest request) {
        log.warn("MissingServletRequestParameterException:", ex);
        StringBuilder msg = new StringBuilder();
        msg.append("缺少必须的[").append(ex.getParameterType()).append("]类型的参数[").append(ex.getParameterName()).append("]");
        return ResponseBuilder.fail(BusinessErrorCode.ILLEGALA_ARGUMENT_EX.getCode(), BusinessErrorCode.ILLEGALA_ARGUMENT_EX.getMessage());
    }

    @ExceptionHandler(NullPointerException.class)
    public ResponseBean<?> nullPointerException(NullPointerException ex, HttpServletRequest request) {
        log.warn("NullPointerException:", ex);
        return ResponseBuilder.fail(BusinessErrorCode.NULL_POINT_EX.getCode(), BusinessErrorCode.NULL_POINT_EX.getMessage());
    }

    @ExceptionHandler(IllegalArgumentException.class)
    public ResponseBean<?> illegalArgumentException(IllegalArgumentException ex, HttpServletRequest request) {
        log.warn("IllegalArgumentException:", ex);
        return ResponseBuilder.fail(BusinessErrorCode.ILLEGALA_ARGUMENT_EX.getCode(), BusinessErrorCode.ILLEGALA_ARGUMENT_EX.getMessage());
    }

    @ExceptionHandler(HttpMediaTypeNotSupportedException.class)
    public ResponseBean<?> httpMediaTypeNotSupportedException(HttpMediaTypeNotSupportedException ex, HttpServletRequest request) {
        log.warn("HttpMediaTypeNotSupportedException:", ex);
        MediaType contentType = ex.getContentType();
        if (contentType != null) {
            StringBuilder msg = new StringBuilder();
            msg.append("请求类型(Content-Type)[").append(contentType.toString()).append("] 与实际接口的请求类型不匹配");
            return ResponseBuilder.fail(BusinessErrorCode.MEDIA_TYPE_EX.getCode(), BusinessErrorCode.MEDIA_TYPE_EX.getMessage());
        }
        return ResponseBuilder.fail(BusinessErrorCode.MEDIA_TYPE_EX.getCode(), BusinessErrorCode.MEDIA_TYPE_EX.getMessage());
    }

    @ExceptionHandler(MissingServletRequestPartException.class)
    public ResponseBean<?> missingServletRequestPartException(MissingServletRequestPartException ex, HttpServletRequest request) {
        log.warn("MissingServletRequestPartException:", ex);
        return ResponseBuilder.fail(BusinessErrorCode.REQUIRED_FILE_PARAM_EX.getCode(), BusinessErrorCode.REQUIRED_FILE_PARAM_EX.getMessage());
    }

    @ExceptionHandler(ServletException.class)
    public ResponseBean<?> servletException(ServletException ex, HttpServletRequest request) {
        log.warn("ServletException:", ex);
        String msg = "UT010016: Not a multi part request";
        if (msg.equalsIgnoreCase(ex.getMessage())) {
            return ResponseBuilder.fail(BusinessErrorCode.REQUIRED_FILE_PARAM_EX.getCode(), BusinessErrorCode.REQUIRED_FILE_PARAM_EX.getMessage());
        }
        return ResponseBuilder.fail(BusinessErrorCode.SYSTEM_BUSY.getCode(), BusinessErrorCode.SYSTEM_BUSY.getMessage());
    }

    @ExceptionHandler(MultipartException.class)
    public ResponseBean<?> multipartException(MultipartException ex, HttpServletRequest request) {
        log.warn("MultipartException:", ex);
        return ResponseBuilder.fail(BusinessErrorCode.REQUIRED_FILE_PARAM_EX.getCode(), BusinessErrorCode.REQUIRED_FILE_PARAM_EX.getMessage());
    }

    /**
     * jsr 规范中的验证异常
     *
     * @param ex
     * @return
     */
    @ExceptionHandler(ConstraintViolationException.class)
    public ResponseBean<?> constraintViolationException(ConstraintViolationException ex, HttpServletRequest request) {
        log.warn("ConstraintViolationException:", ex);
        Set<ConstraintViolation<?>> violations = ex.getConstraintViolations();
        String message = violations.stream().map(ConstraintViolation::getMessage).collect(Collectors.joining(";"));
        return ResponseBuilder.fail(BusinessErrorCode.BASE_VALID_PARAM.getCode(), message);
    }

    /**
     * spring 封装的参数验证异常， 在controller中没有写result参数时，会进入
     *
     * @param ex
     * @return
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseBean<?> methodArgumentNotValidException(MethodArgumentNotValidException ex, HttpServletRequest request) {
        log.warn("MethodArgumentNotValidException:", ex);
        BindingResult bindingResult = ex.getBindingResult();
        return getBindResult(bindingResult);
    }

    @ExceptionHandler(UnexpectedTypeException.class)
    public ResponseBean<?> methodArgumentNotValidException(UnexpectedTypeException ex, HttpServletRequest request) {
        log.warn("UnexpectedTypeException:", ex);
        return ResponseBuilder.fail(ex.getMessage());
    }

    private ResponseBean<?> getBindResult(BindingResult bindingResult) {
        List<FieldError> fieldErrors = bindingResult.getFieldErrors();
        if (log.isDebugEnabled()) {
            for (FieldError error : fieldErrors) {
                log.warn("{} -> {}", error.getDefaultMessage(), error.getDefaultMessage());
            }
        }

        if (fieldErrors.isEmpty()) {
            log.error("validExceptionHandler error fieldErrors is empty");
            ResponseBuilder.fail(BusinessErrorCode.BUSINESS_ERROR.getCode(), "validExceptionHandler error fieldErrors is empty");
        }

        return ResponseBuilder.fail(BusinessErrorCode.BASE_VALID_PARAM.getCode(), fieldErrors.get(0).getDefaultMessage());
    }

    /**
     * 其他异常
     *
     * @param ex
     * @return
     */
    @ExceptionHandler(Exception.class)
    public ResponseBean<?> otherExceptionHandler(Exception ex, HttpServletRequest request) {
        log.warn("Exception:", ex);
        if (ex.getCause() instanceof MyException) {
            return this.handleRRException((MyException) ex.getCause());
        }
        return ResponseBuilder.fail(BusinessErrorCode.SYSTEM_BUSY.getCode(), BusinessErrorCode.SYSTEM_BUSY.getMessage());
    }

    /**
     * 返回状态码:405
     */
    @ExceptionHandler({HttpRequestMethodNotSupportedException.class})
    public ResponseBean<?> handleHttpRequestMethodNotSupportedException(HttpRequestMethodNotSupportedException ex, HttpServletRequest request) {
        log.warn("HttpRequestMethodNotSupportedException:", ex);
        return ResponseBuilder.fail(BusinessErrorCode.METHOD_NOT_ALLOWED.getCode(), BusinessErrorCode.METHOD_NOT_ALLOWED.getMessage());
    }

    @ExceptionHandler(SQLException.class)
    public ResponseBean<?> sqlException(SQLException ex, HttpServletRequest request) {
        log.warn("SQLException:", ex);
        return ResponseBuilder.fail(BusinessErrorCode.SQL_EX.getCode(), BusinessErrorCode.SQL_EX.getMessage());
    }

    @ExceptionHandler(DataIntegrityViolationException.class)
    public ResponseBean<?> dataIntegrityViolationException(DataIntegrityViolationException ex, HttpServletRequest request) {
        log.warn("DataIntegrityViolationException:", ex);
        return ResponseBuilder.fail(BusinessErrorCode.SQL_EX.getCode(), BusinessErrorCode.SQL_EX.getMessage());
    }
}
