package com.ebei.screen.common.response;

import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 构建统一响应对象
 *
 * @author Levi
 */
public class ResponseBuilder<T> {

    /**
     * 成功(默认)
     *
     * @return
     */
    public static <T> ResponseBean<T> ok() {
        return ResponseBean.<T>builder().status(ResponseEnum.OK.status).message(ResponseEnum.OK.message).build();
    }

    /**
     * 失败(默认)
     *
     * @return
     */
    public static <T> ResponseBean<T> fail() {
        return ResponseBean.<T>builder().status(ResponseEnum.FAIL.status).message(ResponseEnum.FAIL.message).build();
    }

    /**
     * 成功(响应信息)
     *
     * @param message 响应信息
     * @return
     */
    public static <T> ResponseBean<T> ok(String message) {
        return ResponseBean.<T>builder().status(ResponseEnum.OK.status).message(message).build();
    }

    /**
     * 失败(响应信息)
     *
     * @param message 响应信息
     * @return
     */
    public static <T> ResponseBean<T> fail(String message) {
        return ResponseBean.<T>builder().status(ResponseEnum.FAIL.status).message(message).build();
    }

    /**
     * 成功(响应码&信息)
     *
     * @param status  响应状态码
     * @param message 响应信息
     * @return
     */
    public static <T> ResponseBean<T> ok(Integer status, String message) {
        return ResponseBean.<T>builder().status(status).message(message).build();
    }

    /**
     * 失败(响应码&信息)
     *
     * @param status  响应状态码
     * @param message 响应信息
     * @return
     */
    public static <T> ResponseBean<T> fail(Integer status, String message) {
        return ok(status, message);
    }

    /**
     * 成功(响应具体内容)
     *
     * @param data 响应具体内容
     * @return
     */
    public static <T> ResponseBean<T> ok(T data) {
        return ResponseBean.<T>builder().status(ResponseEnum.OK.status).message(ResponseEnum.OK.message).data(data).build();
    }

    /**
     * 失败(响应具体内容)
     *
     * @param data 响应具体内容
     * @return
     */
    public static <T> ResponseBean<T> fail(T data) {
        return ResponseBean.<T>builder().status(ResponseEnum.FAIL.status).message(ResponseEnum.FAIL.message).data(data).build();
    }

    /**
     * 成功(响应信息&响应具体内容)
     *
     * @param message 响应信息
     * @param data    响应具体内容
     * @return
     */
    public static <T> ResponseBean<T> ok(String message, T data) {
        return ResponseBean.<T>builder().status(ResponseEnum.OK.status).message(message).data(data).build();
    }

    /**
     * 失败(响应信息&响应具体内容)
     *
     * @param message 响应信息
     * @param data    响应具体内容
     * @return
     */
    public static <T> ResponseBean<T> fail(String message, T data) {
        return ResponseBean.<T>builder().status(ResponseEnum.FAIL.status).message(message).data(data).build();
    }

    /**
     * 成功(响应枚举值)
     *
     * @param responseEnum 响应枚举值
     * @return
     */
    public static <T> ResponseBean<T> ok(ResponseEnum responseEnum) {
        return ResponseBean.<T>builder().status(responseEnum.status).message(responseEnum.message).build();
    }

    /**
     * 失败(响应枚举值)
     *
     * @param responseEnum 响应枚举值
     * @return
     */
    public static <T> ResponseBean<T> fail(ResponseEnum responseEnum) {
        return ok(responseEnum);
    }

    /**
     * 构建分页
     *
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    public static <T, R> ResponseBean<List<R>> ok(List<T> dataList, Function<? super T, ? extends R> function) {
        return ResponseBean.<List<R>>builder().data(buildBean(dataList, function)).status(ResponseEnum.OK.status).message(ResponseEnum.OK.message).build();
    }

    /**
     * 构建分页
     *
     * @param page
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    public static <T, R> ResponseBean<PageBean<R>> ok(PageBean<T> page, Function<? super T, ? extends R> function) {
        return ResponseBean.<PageBean<R>>builder().data(buildPageBean(page, function)).status(ResponseEnum.OK.status).message(ResponseEnum.OK.message).build();
    }

    /**
     * 构建分页
     *
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    private static <T, R> List<R> buildBean(List<T> dataList, Function<? super T, ? extends R> function) {
        return dataList.stream().map(function).collect(Collectors.toList());
    }

    /**
     * 构建分页
     *
     * @param page
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    private static <T, R> PageBean<R> buildPageBean(PageBean<T> page, Function<? super T, ? extends R> function) {
        return PageBean.<R>builder().row(page.getRow().stream().map(function).collect(Collectors.toList()))
                .total(page.getTotal()).build();
    }
}
