package com.ebei.screen.common.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ebei.screen.common.response.ResponseBuilder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.TemporalAdjusters;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * 乱七八糟工具类
 *
 * @author Liu.x
 */
@Slf4j
public class LeviUtils {

    /**
     * 写浏览器数据
     *
     * @param response
     * @param message
     */
    public static void writeJson(HttpServletResponse response, String message) {
        try {
            response.setStatus(HttpServletResponse.SC_OK);
            response.setCharacterEncoding("UTF-8");
            response.setContentType("application/json; charset=utf-8");
            OutputStream writer = response.getOutputStream();
            writer.write(JSON.toJSONString(ResponseBuilder.fail(message)).getBytes(StandardCharsets.UTF_8));
            writer.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取异常的堆栈信息
     *
     * @param ex
     * @return
     */
    public static String getExceptionInfo(Exception ex) {
        StringBuffer errorInfo = new StringBuffer();
        if (ObjectUtil.isNull(ex)) {
            return "";
        }
        try {
            errorInfo.append(ex.toString());
            for (StackTraceElement stackTrace : ex.getStackTrace()) {
                errorInfo.append("\n\t " + stackTrace);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return errorInfo.toString();
    }

    /**
     * 将任意类型变量或数组转集合返回
     *
     * @param
     * @return
     */
    public static <T> List<T> xxToList(T... str) {
        return Arrays.stream(str).collect(Collectors.toList());
    }

    /**
     * 获取指定json的某个路径属性值
     *
     * @param jsonStr json字符串
     * @param path    json路径 例如xx.yy.zz
     * @param t       结果集类型
     * @return
     */
    public static <T> T getJsonFieldOne(String jsonStr, String path, Class<T> t) {
        List<T> jsonField = getJsonFieldMany(jsonStr, path, t);
        if (CollUtil.isNotEmpty(jsonField)) {
            return jsonField.get(0);
        }
        return null;
    }

    /**
     * 获取指定json的某个路径属性值
     *
     * @param jsonStr json字符串
     * @param path    json路径 例如xx.yy.zz 没有传null
     * @param t       结果集类型
     * @return
     */
    public static <T> List<T> getJsonFieldMany(String jsonStr, String path, Class<T> t) {
        Object parseObj = JSON.parse(jsonStr);
        if (StringUtils.isNotBlank(path)) {
            List<String> paths = xxToList(path.split("\\."));
            if (parseObj instanceof JSONObject) {
                JSONObject jsonObject = (JSONObject) parseObj;
                Object nowObj = jsonObject.get(paths.get(0));
                if (paths.size() == 1) {
                    parseObj = nowObj;
                } else {
                    paths.remove(0);
                    return getJsonFieldMany(JSON.toJSONString(nowObj), String.join(".", paths), t);
                }
            } else {
                return null;
            }
        }
        return (parseObj instanceof JSONArray) ? (List<T>) parseObj : xxToList((T) parseObj);
    }

    /**
     * 获取当前时间戳
     *
     * @return
     */
    public static Integer getMonthDays(Integer month) {
        LocalDate localDate = LocalDateTimeUtil.parseDate(LocalDateTime.now().getYear() + "-" + String.format("%02d", month) + "-01");
        return localDate.with(TemporalAdjusters.lastDayOfMonth()).getDayOfMonth();
    }

    /**
     * 集合深拷贝
     *
     * @param srcList
     * @return
     */
    public static <T> List<T> deepCopy(List<T> srcList) {
        ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
        try {
            ObjectOutputStream out = new ObjectOutputStream(byteOut);
            out.writeObject(srcList);
            ByteArrayInputStream byteIn = new ByteArrayInputStream(byteOut.toByteArray());
            ObjectInputStream inStream = new ObjectInputStream(byteIn);
            List<T> destList = (List<T>) inStream.readObject();
            return destList;
        } catch (Exception e) {
            log.info("集合深拷贝出现错误:", e);
        }
        return null;
    }

    /**
     * 秒级时间戳转字符串
     *
     * @param time
     * @return
     */
    public static String convertTime(Object time) {
        return DateUtil.date(Long.parseLong(time + "000")).toString("yyyy-MM-dd HH:mm:ss");
    }

    public static void zipOutput(HttpServletResponse response, List<ByteArrayOutputStream> outs, List<String> fileNames, String zipName) throws IOException {
        File file = new File(zipName + ".zip");
        response.setContentType("application/x-zip-compressed");
        response.addHeader("Access-Control-Expose-Headers", "Content-Disposition");
        response.setHeader("Content-Disposition", "attachment; filename=" + zipName + ".zip");
        try (ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(file))) {
            for (int i = 0; i < outs.size(); i++) {
                ZipEntry zo = new ZipEntry(fileNames.get(i));
                zos.putNextEntry(zo);
                zos.write(outs.get(i).toByteArray());
                zos.closeEntry();
            }
            zos.flush();
            zos.finish();
        }
        int len;
        byte[] buf = new byte[1024];
        try (FileInputStream in = new FileInputStream(file)) {
            while ((len = in.read(buf)) != -1) {
                response.getOutputStream().write(buf, 0, len);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            file.delete();
        }
    }

}
