package com.ebei.screen.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.resource.ResourceUtil;
import cn.hutool.core.map.MapUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ebei.screen.common.constants.CommonConstants;
import com.ebei.screen.common.constants.ParkExceptionType;
import com.ebei.screen.common.constants.ParkType;
import com.ebei.screen.common.constants.PayTypeCode;
import com.ebei.screen.common.exception.MyException;
import com.ebei.screen.common.response.PageBean;
import com.ebei.screen.common.response.ResponseBean;
import com.ebei.screen.common.response.ResponseBuilder;
import com.ebei.screen.common.util.*;
import com.ebei.screen.modules.req.*;
import com.ebei.screen.service.ParkSystemService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStream;
import java.time.Duration;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 车场系统大屏
 *
 * @author Levi.u
 * @date 2021/3/3 11:57
 */
@Slf4j
@Service("parkSystemService")
@Transactional(rollbackFor = {Exception.class})
public class ParkSystemServiceImpl implements ParkSystemService {

    @Autowired
    private MongoTemplate mongoTemplate;

    /**
     * 统一存放到mongo中后续处理
     *
     * @param id     业务id
     * @param params 参数
     * @param flag   false:jlite
     * @return
     */
    @Override
    public synchronized Levi exec(String id, Map params, boolean flag) {
        try {
            // 插入
            mongoTemplate.insert(Levi.by("_id", id).set("data", params), "parkQueue");
            // 更新字典值
            JSONObject obj = mongoTemplate.findById("idDict", JSONObject.class, "parkQueue");
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("idDict"));
            Update update = new Update();
            if (obj != null) {
                update.set("data", obj.getJSONArray("data").fluentAdd(id));
                update.set("count", obj.getIntValue("count") + 1);
            } else {
                update.set("data", new JSONArray().fluentAdd(id));
                update.set("count", 1);
            }
            mongoTemplate.upsert(query, update, "parkQueue");
        } catch (Exception e) {
            log.info("存储队列出现异常:", e);
            return Levi.by(flag ? "resultCode" : "code", flag ? 1 : 0).set(flag ? "message" : "msg", flag ? "处理失败" : "失败");
        }
        return Levi.by(flag ? "resultCode" : "code", flag ? 0 : 1).set(flag ? "message" : "msg", flag ? "处理成功" : "成功");
    }

    /**
     * 处理任务
     *
     * @param size 处理数量
     * @return
     */
    @Override
    public synchronized boolean taskExec(int size) throws InterruptedException {
        boolean result = false;
        JSONObject obj = mongoTemplate.findById("idDict", JSONObject.class, "parkQueue");
        JSONArray data = obj != null ? obj.getJSONArray("data") : new JSONArray();
        if (data != null && data.size() > 0) {
            log.info("即将开始本次处理 当前剩余任务数：" + data.size());
            List<String> ids = (List<String>) (List) data.subList(0, Integer.min(data.size(), size));
            CountDownLatch countDownLatch = new CountDownLatch(Integer.min(data.size(), size));
            ids.forEach(x -> RunnableUtils.start(() -> {
                JSONObject xx = null;
                try {
                    xx = mongoTemplate.findById(x, JSONObject.class, "parkQueue");
                    if (xx == null) {
                        return;
                    }
                    Instant start = Instant.now();
                    JSONObject params = xx.getJSONObject("data");
                    if (x.startsWith("parkInfo")) {
                        this.parkInfo(params);
                    } else if (x.startsWith("parkSpace")) {
                        this.parkSpace(params);
                    } else if (x.startsWith("parkIn")) {
                        this.parkIn(params);
                    } else if (x.startsWith("parkOut")) {
                        this.parkOut(params);
                    } else if (x.startsWith("parkOrder")) {
                        this.parkOrder(params);
                    } else if (x.startsWith("parkCharge")) {
                        this.parkCharge(params);
                    } else if (x.startsWith("parkServiceInfo")) {
                        this.parkServiceInfo(params);
                    }
                    // 处理完成删除当前document
                    mongoTemplate.remove(xx, "parkQueue");
                    Instant end = Instant.now();
                    log.info("当前执行：" + x + " 执行时间：" + Duration.between(start, end).toMillis() + "毫秒");
                } catch (Exception e) {
                    // 删除异常数据
                    mongoTemplate.remove(xx, "parkQueue");
                    log.info("执行队列出现异常:{} 参数:{}", e, xx);
                } finally {
                    countDownLatch.countDown();
                }
            }));
            countDownLatch.await();
            // 删除本批次处理的id
            data.removeAll(ids);
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("idDict"));
            Update update = new Update();
            update.set("data", data);
            update.set("count", data.size());
            mongoTemplate.upsert(query, update, "parkQueue");
            result = true;
            log.info("全部任务处理完成 当前剩余任务数：" + data.size());
        }
        return result;
    }


    /**
     * 车场信息数据推送
     *
     * @param params
     * @return
     */
    @Override
    public void parkInfo(Map params) {
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is("parkInfo"));
        Update update = new Update();
        update.set("data", JSON.parseArray(params.get("dataItems").toString()));
        update.set("pno", params.get("pno"));
        update.set("ts", params.get("ts"));
        mongoTemplate.upsert(query, update, "parkInfo");
    }

    /**
     * 车位信息数据推送
     *
     * @param params
     * @return
     */
    @Override
    public void parkSpace(Map params) {
        Map<String, List<Map>> insertMap = ParkUtils.getInsertMap(params);
        String pno = params.get("pno").toString();
        JSONObject obj = mongoTemplate.findById("parkSpaceLatest", JSONObject.class, "parkSpace");
        JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
        if (MapUtil.isNotEmpty(insertMap)) {
            insertMap.keySet().forEach(key -> {
                List<Map> maps = insertMap.get(key);
                ParkUtils.setDataField(maps, key, "parkSpace", false, null, Levi.by("pno", pno).set("ts", params.get("ts")));
                // 单独存入一条最新的
                data.put(key, maps.get(maps.size() - 1));
            });
        }
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is("parkSpaceLatest"));
        Update update = new Update();
        update.set("data", data);
        mongoTemplate.upsert(query, update, "parkSpace");
    }

    /**
     * 入场信息数据推送
     *
     * @param params
     * @return
     */
    @Override
    public void parkIn(Map params) {
        ParkUtils.parkInOrOut(params, ParkType.IN);
    }

    /**
     * 出场信息数据推送
     *
     * @param params
     * @return
     */
    @Override
    public void parkOut(Map params) {
        ParkUtils.parkInOrOut(params, ParkType.OUT);
    }

    /**
     * 订单信息数据推送
     *
     * @param params
     * @return
     */
    @Override
    public void parkOrder(Map params) {
        Map<String, List<Map>> insertMap = ParkUtils.getInsertMap(params);
        String pno = params.get("pno").toString();
        String format = LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
        if (MapUtil.isNotEmpty(insertMap)) {
            insertMap.keySet().forEach(key -> {
                // 按小时存入数据用于统计
                ParkUtils.setDataField(insertMap.get(key), key, "parkOrder", false, "orderCreateTime", Levi.by("pno", pno).set("ts", params.get("ts")), "orderCreateTime");
                // 按车场存入数据 实时统计当前车场内所有车辆信息
                List<Map> subList = insertMap.get(key);
                Query query2 = new Query();
                query2.addCriteria(Criteria.where("_id").is(key + "_" + format));
                JSONObject obj = mongoTemplate.findById(key + "_" + format, JSONObject.class, "parkHourList");
                JSONObject data = ParkUtils.sortHourList(subList, obj);
                Update update2 = new Update();
                update2.set("pno", pno);
                update2.set("ts", params.get("ts"));
                data.keySet().forEach(x -> update2.set(x, data.get(x)));
                mongoTemplate.upsert(query2, update2, "parkHourList");
            });
        }
    }

    /**
     * 收费记录数据推送
     *
     * @param params
     * @return
     */
    @Override
    public void parkCharge(Map params) {
        List<Map> dataItems = (List<Map>) params.get("dataItems");
        if (CollUtil.isEmpty(dataItems)) {
            return;
        }
        Map<String, List<Map>> insertMap = new HashMap<>(16);
        dataItems.forEach(x -> {
            String parkCode = x.get("parkCode").toString();
            String format = LocalDateTimeUtil.format(ParkUtils.getResultLocalDateTime(x, "feesTime"), DatePattern.PURE_DATE_PATTERN);
            List<Map> maps = new ArrayList<>();
            if (insertMap.containsKey(parkCode + "_" + format)) {
                maps = insertMap.get(parkCode + "_" + format);
            }
            maps.add(x);
            insertMap.put(parkCode + "_" + format, maps);
        });
        if (MapUtil.isNotEmpty(insertMap)) {
            insertMap.keySet().forEach(key -> {
                Query query = new Query();
                query.addCriteria(Criteria.where("_id").is(key));
                Update update = new Update();
                JSONObject obj = mongoTemplate.findById(key, JSONObject.class, "parkCharge");
                JSONArray data = obj != null ? obj.getJSONArray("data") : new JSONArray();
                data.addAll(insertMap.get(key));
                List<Map> feesTime = ParkUtils.sortDateList(data, "feesTime", true);
                update.set("data", feesTime);
                update.set("count", feesTime.size());
                mongoTemplate.upsert(query, update, "parkCharge");
            });
        }
    }

    /**
     * 服务信息数据推送
     *
     * @param params
     */
    @Override
    public void parkServiceInfo(Map params) {
        Map<String, List<Map>> insertMap = ParkUtils.getInsertMap(params, "operateTime");
        String pno = params.get("pno").toString();
        if (MapUtil.isNotEmpty(insertMap)) {
            insertMap.keySet().forEach(key -> {
                Query query = new Query();
                query.addCriteria(Criteria.where("_id").is(key));
                Update update = new Update();
                JSONObject obj = mongoTemplate.findById(key, JSONObject.class, "parkServiceInfo");
                JSONArray data = obj != null ? obj.getJSONArray("data") : new JSONArray();
                data.addAll(insertMap.get(key));
                List<Map> feesTime = ParkUtils.sortDateList(data, "operateTime", true);
                update.set("data", feesTime);
                update.set("pno", pno);
                update.set("ts", params.get("ts"));
                update.set("count", feesTime.size());
                mongoTemplate.upsert(query, update, "parkServiceInfo");
                // 校验是否存在异常数据
                ParkUtils.insertException(key, feesTime, CommonConstants.SERVICE_INFO);
            });
        }
    }

    /**
     * 今日车场流量分析查询
     *
     * @param parkCode
     * @return
     */
    @Override
    public ResponseBean getTodayFlowAnalysis(String parkCode) {
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        List<Integer> indexs = Arrays.asList(0, 3, 6, 9, 12, 15, 18, 21, 23);
        List<String> times = Arrays.asList("00:00", "03:00", "06:00", "09:00", "12:00", "15:00", "18:00", "21:00", "23:59");
        AtomicReference<Integer> inCountResult = new AtomicReference<>(0);
        AtomicReference<Integer> inCountOldResult = new AtomicReference<>(0);
        AtomicReference<Integer> outCountResult = new AtomicReference<>(0);
        AtomicReference<Integer> outCountOldResult = new AtomicReference<>(0);
        AtomicReference<Integer> exceptionCountResult = new AtomicReference<>(0);
        AtomicReference<Integer> exceptionCountOldResult = new AtomicReference<>(0);
        List<Levi> result = new ArrayList<>();
        times.forEach(x -> result.add(Levi.by("time", x).set("in", 0).set("out", 0).set("exception", 0)));
        parkCodeList.forEach(code -> {
            String format = LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
            String formatOld = LocalDateTimeUtil.format(LocalDateTimeUtil.offset(LocalDateTime.now(), -1, ChronoUnit.DAYS), DatePattern.PURE_DATE_PATTERN);
            JSONObject inObj = mongoTemplate.findById(code + "_" + format, JSONObject.class, ParkType.IN.getCode());
            JSONObject outObj = mongoTemplate.findById(code + "_" + format, JSONObject.class, ParkType.OUT.getCode());
            JSONObject exceptionObj = mongoTemplate.findById(code + "_" + format, JSONObject.class, ParkType.EXCEPTION.getCode());
            JSONObject exceptionData = exceptionObj != null ? exceptionObj.getJSONObject("data") : null;
            JSONObject exceptionObjOld = mongoTemplate.findById(code + "_" + formatOld, JSONObject.class, ParkType.EXCEPTION.getCode());
            ParkExceptionType.codeList().forEach(x -> exceptionCountResult.set((exceptionObj == null ? 0 : (exceptionObj.getDouble(x + "Count") == null ? 0 : exceptionObj.getDouble(x + "Count").intValue())) + exceptionCountResult.get()));
            ParkExceptionType.codeList().forEach(x -> exceptionCountOldResult.set((exceptionObjOld == null ? 0 : (exceptionObjOld.getDouble(x + "Count") == null ? 0 : exceptionObjOld.getDouble(x + "Count").intValue())) + exceptionCountOldResult.get()));
            System.out.println("当前车场:" + code + " 异常数量:" + exceptionCountResult);
            List<Map> exceptionResult = new ArrayList<>();
            if (exceptionData != null) {
                exceptionData.keySet().forEach(x -> exceptionResult.addAll((List<Map>) exceptionData.get(x)));
            }
            for (int i = 0; i < indexs.size(); i++) {
                Levi levi = result.get(i);
                levi.set("exception", levi.getInt("exception") + ParkUtils.getExceptionDataListByHour(exceptionResult, indexs.get(i)).size());
            }
            if (inObj == null && outObj == null) {
                return;
            }
            JSONArray inObjList = inObj != null ? inObj.getJSONArray("data") : null;
            JSONObject inObjOld = mongoTemplate.findById(code + "_" + formatOld, JSONObject.class, ParkType.IN.getCode());
            JSONArray outObjList = outObj != null ? outObj.getJSONArray("data") : null;
            JSONObject outObjOld = mongoTemplate.findById(code + "_" + formatOld, JSONObject.class, ParkType.OUT.getCode());
            Double inCount = inObj == null ? 0 : inObj.getDouble("count");
            Double inCountOld = inObjOld == null ? 0 : inObjOld.getDouble("count");
            Double outCount = outObj == null ? 0 : outObj.getDouble("count");
            Double outCountOld = outObjOld == null ? 0 : outObjOld.getDouble("count");
            inCountResult.set(inCount.intValue() + inCountResult.get());
            inCountOldResult.set(inCountOld.intValue() + inCountOldResult.get());
            outCountResult.set(outCount.intValue() + outCountResult.get());
            outCountOldResult.set(outCountOld.intValue() + outCountOldResult.get());
            for (int i = 0; i < indexs.size(); i++) {
                Levi levi = result.get(i);
                levi.set("in", levi.getInt("in") + (inObjList == null ? 0 : inObjList.getJSONObject(indexs.get(i)).getJSONArray("data").size()));
                levi.set("out", levi.getInt("out") + (outObjList == null ? 0 : outObjList.getJSONObject(indexs.get(i)).getJSONArray("data").size()));
            }
        });
        return ResponseBuilder.ok(Levi.by("inCount", inCountResult.get()).set("outCount", outCountResult.get()).set("exceptionCount", exceptionCountResult.get()).set("inPercent", EnergyUtils.getPercent(inCountResult.get().doubleValue(), inCountOldResult.get().doubleValue())).set("outPercent", EnergyUtils.getPercent(outCountResult.get().doubleValue(), outCountOldResult.get().doubleValue())).set("exceptionPercent", EnergyUtils.getPercent(exceptionCountResult.get().doubleValue(), exceptionCountOldResult.get().doubleValue())).set("list", result));
    }

    /**
     * 根据停车场编号获取当前车场入场数
     *
     * @param parkCode 停车场编号
     * @return
     */
    private Double getCurrentParkInNum(String parkCode) {
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        AtomicReference<Integer> inCountResult = new AtomicReference<>(0);
        parkCodeList.forEach(code -> {
            String format = LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
            JSONObject inObj = mongoTemplate.findById(code + "_" + format, JSONObject.class, ParkType.IN.getCode());
            Double inCount = inObj == null ? 0 : inObj.getDouble("count");
            inCountResult.set(inCount.intValue() + inCountResult.get());
        });
        return inCountResult.get().doubleValue();
    }

    /**
     * 今日车场流量分析分页查询折线
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean getTodayFlowAnalysisByLine(FlowAnalysisReq params) {
        String parkCode = params.getParkCode();
        Integer type = params.getType();
        Integer timeIndex = params.getTimeIndex();
        String date = params.getDate();
        List<Integer> indexs = Arrays.asList(0, 3, 6, 9, 12, 15, 18, 21, 23);
        if (!indexs.contains(timeIndex)) {
            return ResponseBuilder.fail("暂不支持该时间段查询");
        }
        if (type != 1 && type != 2 && type != 3) {
            return ResponseBuilder.fail("类型只能为入场、出场、异常");
        }
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        String format = StringUtils.isNotBlank(date) ? date.replace("-", "") : LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
        List<Map> results = new ArrayList<>();
        parkCodeList.forEach(code -> {
            JSONObject obj = mongoTemplate.findById(code + "_" + format, JSONObject.class, ParkType.getTypeByNo(type).getCode());
            if (obj == null) {
                return;
            }
            // 异常类型
            if (CommonConstants.EXCEPTION_TYPE.equals(type)) {
                JSONObject data = obj.getJSONObject("data");
                data.keySet().forEach(x -> results.addAll(ParkUtils.getExceptionDataListByHour((List<Map>) data.get(x), timeIndex)));
            } else {
                List<Map> maps = obj.getJSONArray("data").toJavaList(Map.class);
                results.addAll((List<Map>) maps.get(timeIndex).get("data"));
            }
        });
        List<Map> maps = ParkUtils.sortDateList(results, ParkType.getTypeByNo(type).getSort(), true);
        Supplier<Stream<Map>> resStream = () -> maps.stream();
        List<Map> resultsList = resStream.get().skip((params.getPageNum() - 1) * params.getPageSize()).limit(params.getPageSize()).collect(Collectors.toList());
        return ResponseBuilder.ok(PageBean.<Map>builder().total(resStream.get().count()).row(resultsList).build());
    }


    /**
     * 今日车场流量分析分页查询详情
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean getTodayFlowAnalysisByPage(FlowAnalysisReq params) {
        String parkCode = params.getParkCode();
        Integer type = params.getType();
        String date = params.getDate();
        if (type != 1 && type != 2 && type != 3) {
            return ResponseBuilder.fail("类型只能为入场、出场、异常");
        }
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        String format = StringUtils.isNotBlank(date) ? date.replace("-", "") : LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
        List<Map> results = new ArrayList<>();
        parkCodeList.forEach(code -> {
            JSONObject obj = mongoTemplate.findById(code + "_" + format, JSONObject.class, ParkType.getTypeByNo(type).getCode());
            if (obj == null) {
                return;
            }
            // 异常类型
            if (CommonConstants.EXCEPTION_TYPE.equals(type)) {
                JSONObject data = obj.getJSONObject("data");
                data.keySet().forEach(x -> results.addAll((List<Map>) data.get(x)));
            } else {
                List<Map> maps = obj.getJSONArray("data").toJavaList(Map.class);
                maps.forEach(x -> results.addAll((List<Map>) x.get("data")));
            }
        });
        List<Map> maps = ParkUtils.sortDateList(results, ParkType.getTypeByNo(type).getSort(), true);
        Supplier<Stream<Map>> resStream = () -> maps.stream();
        List<Map> resultsList = resStream.get().skip((params.getPageNum() - 1) * params.getPageSize()).limit(params.getPageSize()).collect(Collectors.toList());
        if (CommonConstants.EXCEPTION_TYPE.equals(type)) {
            resultsList.forEach(x -> x.put("imageUrl", x.get("imageUrl") == null ? null : ParkUtils.getImgUrlById(x.get("imageUrl").toString())));
        } else {
            resultsList.forEach(x -> x.put("imageUrl", ParkUtils.getImgUrlById(x.get(ParkType.getTypeByNo(type).getImage()).toString())));
        }
        return ResponseBuilder.ok(PageBean.<Map>builder().total(resStream.get().count()).row(resultsList).build());
    }


    /**
     * 入场车辆通车时间段统计
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean inCarAnalysis(InCarAnalysisReq params) {
        String parkCode = params.getParkCode();
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        if (StringUtils.isBlank(params.getDate())) {
            return ResponseBuilder.fail("未指定查询日期");
        }
        List<Levi> result = new ArrayList<>();
        Levi times = Levi.by("昨日22点至今日9点", 22).set("9点至10点", 9).set("10点至11点", 10).set("11点至12点", 11).set("12点至13点", 12).set("13点至14点", 13).set("14点至15点", 14).set("15点至16点", 15).set("16点至17点", 16).set("17点至18点", 17).set("18点至19点", 18).set("19点至20点", 19).set("20点至21点", 20).set("21点至22点", 21);
        times.keySet().forEach(key -> result.add(Levi.by("time", key).set("count", 0).set("id", times.getInt(key))));
        parkCodeList.forEach(code -> {
            LocalDateTime day = LocalDateTimeUtil.parse(params.getDate(), DatePattern.NORM_DATE_PATTERN);
            LocalDateTime yes = LocalDateTimeUtil.offset(day, -1, ChronoUnit.DAYS);
            String dayFormat = LocalDateTimeUtil.format(day, DatePattern.PURE_DATE_PATTERN);
            String yesFormat = LocalDateTimeUtil.format(yes, DatePattern.PURE_DATE_PATTERN);
            JSONObject dayObj = mongoTemplate.findById(code + "_" + dayFormat, JSONObject.class, "parkIn");
            JSONObject yesObj = mongoTemplate.findById(code + "_" + yesFormat, JSONObject.class, "parkIn");
            if (dayObj == null && yesObj == null) {
                return;
            }
            JSONArray dayObjList = dayObj != null ? dayObj.getJSONArray("data") : null;
            JSONArray yesObjList = yesObj != null ? yesObj.getJSONArray("data") : null;
            Integer first = yesObjList == null ? 0 : (yesObjList.getJSONObject(22).getInteger("count") + yesObjList.getJSONObject(23).getInteger("count"));
            // 封装昨日22点至今日9点
            for (int i = 0; i < 9; i++) {
                first += (dayObjList == null ? 0 : dayObjList.getJSONObject(i).getInteger("count"));
            }
            result.get(0).set("count", result.get(0).getInt("count") + first);
            // 封装剩余时间
            for (int i = 1; i < result.size(); i++) {
                result.get(i).set("count", result.get(i).getInt("count") + (dayObjList == null ? 0 : dayObjList.getJSONObject(i + 8).getInteger("count")));
            }
        });
        return ResponseBuilder.ok(result);
    }

    /**
     * 入场车辆通车时间段统计分页查询详情
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean inCarAnalysisByPage(InCarAnalysisPageReq params) {
        String parkCode = params.getParkCode();
        Integer dateId = params.getDateId();
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        if (dateId == null || dateId < 9 || dateId > 22) {
            return ResponseBuilder.fail("日期ID不存在");
        }
        if (StringUtils.isBlank(params.getDate())) {
            return ResponseBuilder.fail("未指定查询日期");
        }
        List<Map> result = new ArrayList<>();
        parkCodeList.forEach(code -> {
            LocalDateTime day = LocalDateTimeUtil.parse(params.getDate(), DatePattern.NORM_DATE_PATTERN);
            String dayFormat = LocalDateTimeUtil.format(day, DatePattern.PURE_DATE_PATTERN);
            JSONObject dayObj = mongoTemplate.findById(code + "_" + dayFormat, JSONObject.class, "parkIn");
            JSONArray dayObjList = dayObj != null ? dayObj.getJSONArray("data") : null;
            if (dateId != 22 && CollUtil.isNotEmpty(dayObjList)) {
                result.addAll(dayObjList.getJSONObject(dateId).getJSONArray("data") == null ? new ArrayList<>() : dayObjList.getJSONObject(dateId).getJSONArray("data").toJavaList(Map.class));
                return;
            }
            LocalDateTime yes = LocalDateTimeUtil.offset(day, -1, ChronoUnit.DAYS);
            String yesFormat = LocalDateTimeUtil.format(yes, DatePattern.PURE_DATE_PATTERN);
            JSONObject yesObj = mongoTemplate.findById(code + "_" + yesFormat, JSONObject.class, "parkIn");
            JSONArray yesObjList = yesObj != null ? yesObj.getJSONArray("data") : null;
            if (CollUtil.isNotEmpty(yesObjList)) {
                result.addAll(yesObjList.getJSONObject(22).getJSONArray("data") == null ? new ArrayList<>() : yesObjList.getJSONObject(22).getJSONArray("data").toJavaList(Map.class));
                result.addAll(yesObjList.getJSONObject(23).getJSONArray("data") == null ? new ArrayList<>() : yesObjList.getJSONObject(22).getJSONArray("data").toJavaList(Map.class));
            }
            // 封装昨日22点至今日9点
            if (CollUtil.isNotEmpty(dayObjList)) {
                for (int i = 0; i < 9; i++) {
                    result.addAll(dayObjList.getJSONObject(i).getJSONArray("data") == null ? new ArrayList<>() : dayObjList.getJSONObject(i).getJSONArray("data").toJavaList(Map.class));
                }
            }
        });
        List<Map> maps = ParkUtils.sortDateList(result, ParkType.IN.getSort(), true);
        Supplier<Stream<Map>> resStream = () -> maps.stream();
        List<Map> resultsList = resStream.get().skip((params.getPageNum() - 1) * params.getPageSize()).limit(params.getPageSize()).collect(Collectors.toList());
        return ResponseBuilder.ok(PageBean.<Map>builder().total(resStream.get().count()).row(resultsList).build());
    }

    /**
     * 今日停车时长统计
     *
     * @param parkCode
     * @return
     */
    @Override
    public ResponseBean parkingHoursToday(String parkCode) {
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        List<String> idsStr = Arrays.asList("1-3h", "3-6h", "6-9h", "9-12h", "12h以上");
        List<String> ids = Arrays.asList("1", "3", "6", "9", "12");
        Levi<String, Integer> result = Levi.create();
        List<Levi> results = new ArrayList<>();
        ids.forEach(x -> result.set("count" + x, 0));
        parkCodeList.forEach(code -> {
            String format = LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
            JSONObject obj = mongoTemplate.findById(code + "_" + format, JSONObject.class, "parkHourList");
            if (obj == null) {
                return;
            }
            ids.forEach(x -> result.set("count" + x, result.get("count" + x) + obj.getInteger("count" + x)));
        });
        for (int i = 0; i < ids.size(); i++) {
            results.add(Levi.by("time", idsStr.get(i)).set("value", result.getInt("count" + ids.get(i))));
        }
        return ResponseBuilder.ok(results);
    }

    /**
     * 今日停车时长统计分页查询详情
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean parkingHoursTodayByPage(ParkHoursPageReq params) {
        String parkCode = params.getParkCode();
        String hour = params.getHour();
        String date = params.getDate();
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        List<String> ids = Arrays.asList("1", "3", "6", "9", "12");
        if (StringUtils.isBlank(hour) || !ids.contains(hour)) {
            return ResponseBuilder.fail("不支持的时间");
        }
        String format = StringUtils.isNotBlank(date) ? date.replace("-", "") : LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
        List<Map> result = new ArrayList<>();
        parkCodeList.forEach(code -> {
            JSONObject obj = mongoTemplate.findById(code + "_" + format, JSONObject.class, "parkHourList");
            if (obj == null) {
                return;
            }
            JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
            JSONArray hourData = data != null ? data.getJSONArray(hour) : new JSONArray();
            result.addAll(hourData != null ? hourData.toJavaList(Map.class) : new ArrayList<>());
        });
        List<Map> maps = ParkUtils.sortDateList(result, "orderCreateTime", true);
        // 处理单位
        maps.forEach(x -> {
            x.put("serviceFeeTime", x.get("serviceFeeTime") == null ? 0 : String.format("%.2f", Double.parseDouble(x.get("serviceFeeTime").toString()) / 3600) + "小时");
            x.put("totalFee", x.get("totalFee") == null ? 0 : String.format("%.2f", Double.parseDouble(x.get("totalFee").toString()) / 100) + "元");
        });
        Supplier<Stream<Map>> resStream = () -> maps.stream();
        List<Map> resultsList = resStream.get().skip((params.getPageNum() - 1) * params.getPageSize()).limit(params.getPageSize()).collect(Collectors.toList());
        return ResponseBuilder.ok(PageBean.<Map>builder().total(resStream.get().count()).row(resultsList).build());
    }

    /**
     * 关键指标查询
     *
     * @param parkCode
     * @return
     */
    @Override
    public ResponseBean keyIndex(String parkCode) {
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        List<Levi> list = new ArrayList<>();
        Levi result = Levi.by("cwzzl", "0.00").set("tcwds", "0").set("yzcws", "0").set("kxcws", "0");
        JSONObject obj = mongoTemplate.findById("parkSpaceLatest", JSONObject.class, "parkSpace");
        if (obj == null) {
            return ResponseBuilder.ok(result);
        }
        JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
        AtomicReference<Integer> inCountResult = new AtomicReference<>(0);
        parkCodeList.forEach(code -> {
            if (data.getJSONObject(code) != null) {
                Double currentParkInNum = getCurrentParkInNum(code);
                JSONObject jsonObject = data.getJSONObject(code);
                Levi set = Levi.by("parkName", jsonObject.getString("parkName")).set("tcwds", jsonObject.getInteger("totalSpace")).set("kxcws", jsonObject.getInteger("remainSpace")).set("yzcws", jsonObject.getInteger("totalSpace") - jsonObject.getInteger("remainSpace"));
                set.set("cwzzl", EnergyUtils.divide(currentParkInNum, Double.parseDouble(set.getStr("tcwds")), 2));
                list.add(set);
                result.set("tcwds", (Integer.parseInt(result.getStr("tcwds")) + jsonObject.getInteger("totalSpace") + ""))
                        .set("kxcws", (Integer.parseInt(result.getStr("kxcws")) + jsonObject.getInteger("remainSpace") + ""));
                result.set("yzcws", (Integer.parseInt(result.getStr("tcwds")) - Integer.parseInt(result.getStr("kxcws"))) + "");
                inCountResult.set(currentParkInNum.intValue() + inCountResult.get());
            }
        });
        result.set("cwzzl", EnergyUtils.divide(inCountResult.get().doubleValue(), Double.parseDouble(result.getStr("tcwds")), 2));
        result.set("list", list);
        return ResponseBuilder.ok(result);
    }


    /**
     * 关键指标分页查询详情
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean keyIndexByPage(KeyIndexPageReq params) {
        String parkCode = params.getParkCode();
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        Levi<String, String> parkCodeMap = ParkUtils.initParkCodeList();
        List<Map> list = new ArrayList<>();
        JSONObject obj = mongoTemplate.findById("parkSpaceLatest", JSONObject.class, "parkSpace");
        if (obj != null) {
            JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
            parkCodeList.forEach(code -> {
                if (data.getJSONObject(code) != null) {
                    JSONObject jsonObject = data.getJSONObject(code);
                    Levi set = Levi.by("parkName", parkCodeMap.get(code)).set("tcwds", jsonObject.getInteger("totalSpace")).set("kxcws", jsonObject.getInteger("remainSpace")).set("yzcws", jsonObject.getInteger("totalSpace") - jsonObject.getInteger("remainSpace"));
                    set.set("cwzzl", EnergyUtils.divide(getCurrentParkInNum(code), Double.parseDouble(set.getStr("tcwds")), 2));
                    list.add(set);
                } else {
                    list.add(Levi.by("parkName", parkCodeMap.get(code)).set("tcwds", 0).set("kxcws", 0).set("yzcws", 0).set("cwzzl", "0.00"));
                }
            });
        }
        Supplier<Stream<Map>> resStream = () -> list.stream();
        List<Map> resultsList = resStream.get().skip((params.getPageNum() - 1) * params.getPageSize()).limit(params.getPageSize()).collect(Collectors.toList());
        return ResponseBuilder.ok(PageBean.<Map>builder().total(resStream.get().count()).row(resultsList).build());
    }

    /**
     * 异常趋势统计查询
     *
     * @param parkCode
     * @return
     */
    @Override
    public ResponseBean exceptionAnalysis(String parkCode) {
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        List<Levi> list = new ArrayList<>();
        Levi result = Levi.create();
        ParkExceptionType.codeList().forEach(x -> result.set(x, 0));
        parkCodeList.forEach(code -> {
            String format = LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
            JSONObject obj = mongoTemplate.findById(code + "_" + format, JSONObject.class, ParkType.EXCEPTION.getCode());
            if (obj == null) {
                return;
            }
            ParkExceptionType.codeList().forEach(x -> result.set(x, result.getInt(x) + (obj.getInteger(x + "Count") == null ? 0 : obj.getInteger(x + "Count"))));
        });
        result.keySet().forEach(x -> list.add(Levi.by("name", ParkExceptionType.getTypeByCode(x.toString()).getName()).set("code", x).set("count", result.getInt(x))));
        return ResponseBuilder.ok(list);
    }

    /**
     * 异常趋势统计分页查询详情
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean exceptionAnalysisByPage(ExceptionTrendReq params) {
        String parkCode = params.getParkCode();
        String paramsCode = params.getCode();
        String date = params.getDate();
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        if (!ParkExceptionType.codeList().contains(paramsCode)) {
            return ResponseBuilder.fail("异常编号错误");
        }
        String format = StringUtils.isNotBlank(date) ? date.replace("-", "") : LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
        List<Map> result = new ArrayList<>();
        parkCodeList.forEach(code -> {
            JSONObject obj = mongoTemplate.findById(code + "_" + format, JSONObject.class, ParkType.EXCEPTION.getCode());
            if (obj == null) {
                return;
            }
            JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
            JSONArray dataList = data != null ? data.getJSONArray(paramsCode) : new JSONArray();
            result.addAll(dataList != null ? dataList.toJavaList(Map.class) : new ArrayList<>());
        });
        List<Map> maps = ParkUtils.sortDateList(result, ParkType.EXCEPTION.getSort(), true);
        Supplier<Stream<Map>> resStream = () -> maps.stream();
        List<Map> resultsList = resStream.get().skip((params.getPageNum() - 1) * params.getPageSize()).limit(params.getPageSize()).collect(Collectors.toList());
        resultsList.forEach(x -> x.put("imageUrl", x.get("imageUrl") == null ? null : ParkUtils.getImgUrlById(x.get("imageUrl").toString())));
        return ResponseBuilder.ok(PageBean.<Map>builder().total(resStream.get().count()).row(resultsList).build());
    }

    /**
     * 根据集合名称修复所有指定日期后的数字
     *
     * @param collection
     * @param date
     * @return
     */
    @Override
    public ResponseBean fixDate(String collection, String date) {
        List<String> dateIds = ParkUtils.getDateIds(date);
        List<String> list = new ArrayList<>();
        dateIds.forEach(dateId -> {
            List<String> parkCodeList = ParkUtils.getParkCodeList();
            parkCodeList.forEach(parkCode -> {
                // 修复业务数据
                Map dayMap = mongoTemplate.findById(parkCode + "_" + dateId, Map.class, collection);
                if (MapUtil.isNotEmpty(dayMap)) {
                    list.add(parkCode + "_" + dateId);
                    Query query = new Query();
                    query.addCriteria(Criteria.where("_id").is(parkCode + "_" + dateId));
                    Update update = new Update();
                    List<Map> hourList = (List<Map>) dayMap.get("data");
                    AtomicReference<Integer> countNum = new AtomicReference<>(0);
                    hourList.forEach(hourMap -> {
                        List<Map> dataList = (List<Map>) hourMap.get("data");
                        if (CollUtil.isNotEmpty(dataList)) {
                            countNum.set(countNum.get() + dataList.size());
                        }
                        hourMap.put("count", dataList.size());
                    });
                    update.set("count", countNum.get());
                    update.set("data", dayMap.get("data"));
                    mongoTemplate.upsert(query, update, collection);
                }
                // 修复异常数字
                Map dayMapException = mongoTemplate.findById(parkCode + "_" + dateId, Map.class, "parkException");
                if (MapUtil.isNotEmpty(dayMapException)) {
                    Query query = new Query();
                    query.addCriteria(Criteria.where("_id").is(parkCode + "_" + dateId));
                    Update update = new Update();
                    Map<String, List> exceptionMap = (Map<String, List>) dayMapException.get("data");
                    exceptionMap.keySet().forEach(key -> update.set(key + "Count", exceptionMap.get(key).size()));
                    mongoTemplate.upsert(query, update, "parkException");
                }
            });
        });
        return ResponseBuilder.ok("修复车场完成", list);
    }

    /**
     * 异常信息导出
     *
     * @param params
     * @param response
     */
    @Override
    public void exportException(ExceptionTrendReq params, HttpServletResponse response) {
        String parkCode = params.getParkCode();
        String paramsCode = params.getCode();
        String date = params.getDate();
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        if (!ParkExceptionType.codeList().contains(paramsCode)) {
            throw new MyException("异常编号错误");
        }
        String format = StringUtils.isNotBlank(date) ? date.replace("-", "") : LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
        CountDownLatch countDownLatch = new CountDownLatch(parkCodeList.size());
        List<ByteArrayOutputStream> streams = new ArrayList<>();
        List<String> excelNames = new ArrayList<>();
        parkCodeList.forEach(code -> RunnableUtils.start(() -> {
            try {
                JSONObject obj = mongoTemplate.findById(code + "_" + format, JSONObject.class, ParkType.EXCEPTION.getCode());
                if (obj == null) {
                    return;
                }
                String fileName = ParkUtils.getParkNameByCode(code) + ".xlsx";
                JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
                JSONArray dataList = data != null ? data.getJSONArray(paramsCode) : new JSONArray();
                List<Map> objects = dataList != null ? dataList.toJavaList(Map.class) : new ArrayList<>();
                List<Map> maps = ParkUtils.sortDateList(objects, ParkType.EXCEPTION.getSort(), true);
                if (CollUtil.isNotEmpty(maps)) {
                    maps.forEach(x -> x.put("photoUrl", ParkUtils.getExcelPhotoUrl(x.get("imageUrl") == null ? (x.get("outCarPhoto") == null ? x.get("inCarPhoto") : x.get("outCarPhoto")) : x.get("imageUrl"))));
                }
                InputStream inputStream = ResourceUtil.getStream("temp/exception.xlsx");
                File ex = File.createTempFile("exception", ".xlsx");
                FileUtil.writeFromStream(inputStream, ex);
                ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
                EasyExcel.write(outputStream).withTemplate(ex).sheet("异常信息").doFill(maps);
                excelNames.add(fileName);
                streams.add(outputStream);
            } catch (Exception e) {
                log.info("异常信息报表导出出现异常:{}", e);
            } finally {
                countDownLatch.countDown();
            }
        }));
        try {
            countDownLatch.await();
            LeviUtils.zipOutput(response, streams, excelNames, "异常信息导出");
        } catch (Exception e) {
            log.info("异常信息压缩zip异常:{}", e);
        }
    }

    /**
     * 不同收费方式停车统计
     *
     * @param parkCode
     * @return
     */
    @Override
    public ResponseBean differentPay(String parkCode) {
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        }
        List<Map> result = new ArrayList<>();
        parkCodeList.forEach(code -> {
            String format = LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
            JSONObject obj = mongoTemplate.findById(code + "_" + format, JSONObject.class, "parkCharge");
            if (obj == null) {
                return;
            }
            List<Map> dataList = obj != null ? obj.getJSONArray("data").toJavaList(Map.class) : new JSONArray().toJavaList(Map.class);
            result.addAll(dataList);
        });
        Map<String, List<Map>> payType = result.stream().collect(Collectors.groupingBy(x -> x.get("payType").toString()));
        result.clear();
        Arrays.stream(PayTypeCode.values()).forEach(x -> result.add(Levi.by("name", x.getType()).set("code", x.getCode()).set("count", payType.get(x.getCode()) == null ? 0 : payType.get(x.getCode()).size())));
        return ResponseBuilder.ok(result);
    }

    /**
     * 不同收费方式停车统计分页查询详情
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean differentPayByPage(DifferentPayPageReq params) {
        String parkCode = params.getParkCode();
        String payType = params.getPayType();
        String date = params.getDate();
        String split = "否";
        List<String> parkCodeList = Arrays.asList(parkCode);
        if (StringUtils.isBlank(parkCode)) {
            parkCodeList = ParkUtils.getParkCodeList();
        } else {
            // 判断当前查询的parkCode是否需要拆分
            JSONObject splitObj = mongoTemplate.findById(CommonConstants.PARK_SPLIT, JSONObject.class, CommonConstants.PARK_SPLIT);
            List<String> splitIds = splitObj != null ? splitObj.getJSONArray("data").toJavaList(String.class) : new JSONArray().toJavaList(String.class);
            if (CollUtil.isNotEmpty(splitIds) && splitIds.contains(parkCode)) {
                split = "是";
            }
        }
        String format = StringUtils.isNotBlank(date) ? date.replace("-", "") : LocalDateTimeUtil.format(LocalDateTime.now(), DatePattern.PURE_DATE_PATTERN);
        List<Map> result = new ArrayList<>();
        parkCodeList.forEach(code -> {
            JSONObject obj = mongoTemplate.findById(code + "_" + format, JSONObject.class, "parkCharge");
            if (obj == null) {
                return;
            }
            List<Map> dataList = obj != null ? obj.getJSONArray("data").toJavaList(Map.class) : new JSONArray().toJavaList(Map.class);
            result.addAll(dataList);
        });
        Map<String, List<Map>> map = result.stream().collect(Collectors.groupingBy(x -> x.get("payType").toString()));
        List<Map> dataList = map.get(payType);
        dataList.forEach(x -> x.put("payTypeName", PayTypeCode.getTypeByCode(x.get("payType").toString())));
        Supplier<Stream<Map>> resStream = () -> dataList.stream();
        List<Map> resultsList = resStream.get().skip((params.getPageNum() - 1) * params.getPageSize()).limit(params.getPageSize()).collect(Collectors.toList());
        return ResponseBuilder.ok(Levi.by("total", resStream.get().count()).set("row", resultsList).set("split", split));
    }

    /**
     * 查询已配置的crm项目列表
     *
     * @return
     */
    @Override
    public ResponseBean getShowProjectList() {
        JSONObject obj = mongoTemplate.findById("relevanceParkProject", JSONObject.class, "relevanceParkProject");
        JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
        JSONObject crmProjectInfo = EbaUtils.crmProjectInfo;
        List<Map> list = new ArrayList<>();
        data.keySet().forEach(x -> list.add(Levi.by("projectId", x).set("projectName", crmProjectInfo.get(x)).set("parkCode", data.getString(x))));
        return ResponseBuilder.ok(ParkUtils.sortList(list));
    }

    /**
     * 删除已有的关联项目
     *
     * @param ebeiProjectId
     * @return
     */
    @Override
    public ResponseBean deleteRelevance(String ebeiProjectId) {
        JSONObject obj = mongoTemplate.findById("relevanceParkProject", JSONObject.class, "relevanceParkProject");
        JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
        data.remove(ebeiProjectId);
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is("relevanceParkProject"));
        Update update = new Update();
        update.set("data", data);
        mongoTemplate.upsert(query, update, "relevanceParkProject");
        return ResponseBuilder.ok("删除成功");
    }

    /**
     * 获取项目列表
     *
     * @return
     */
    @Override
    public ResponseBean getProjectList() {
        Levi<String, String> map = ParkUtils.initParkCodeList();
        List<Levi> result = map.keySet().stream().map(key -> Levi.by("parkName", map.get(key)).set("parkCode", key)).collect(Collectors.toList());
        return ResponseBuilder.ok(result);
    }

}
